@@ -1,6 +1,6 @@
 package perfSONAR_PS::RequestHandler;
 
-our $VERSION = 0.04;
+our $VERSION = 0.05;
 
 =head1 NAME
 
@@ -57,10 +57,11 @@ sub new($) {
     message.
 =cut
 sub addFullMessageHandler($$$$) {
-    my ($self, $messageType, $service) = validate_pos(@_,
+    my ($self, $messageType, $service, $handle_chaining) = validate_pos(@_,
                 1,
                 { type => SCALAR },
                 { can => 'handleMessage'},
+                { type => SCALAR | UNDEF, optional => 1},
             );
 
     my $logger = get_logger("perfSONAR_PS::RequestHandler");
@@ -91,10 +92,11 @@ sub addFullMessageHandler($$$$) {
     been handled.
 =cut
 sub addMessageHandler($$$) {
-    my ($self, $messageType, $service) = validate_pos(@_,
+    my ($self, $messageType, $service, $handle_chaining) = validate_pos(@_,
                 1,
                 { type => SCALAR },
                 { can => [ 'handleMessageBegin', 'handleMessageEnd', 'handleEvent' ]}
+                { type => SCALAR | UNDEF, optional => 1},
             );
 
     my $logger = get_logger("perfSONAR_PS::RequestHandler");
@@ -119,11 +121,12 @@ sub addMessageHandler($$$) {
     specified type found in a message of the specified type.
 =cut
 sub addEventHandler($$$$) {
-    my ($self, $messageType, $eventType, $service) = validate_pos(@_,
+    my ($self, $messageType, $eventType, $service, $handle_chaining) = validate_pos(@_,
                 1,
                 { type => SCALAR },
                 { type => SCALAR },
                 { can => [ 'handleEvent' ]}
+                { type => SCALAR | UNDEF, optional => 1},
             );
 
     my $logger = get_logger("perfSONAR_PS::RequestHandler");
@@ -153,11 +156,12 @@ sub addEventHandler($$$$) {
     type.
 =cut
 sub addEventHandler_Regex($$$$) {
-    my ($self, $messageType, $eventRegex, $service) = validate_pos(@_,
+    my ($self, $messageType, $eventRegex, $service, $handle_chaining) = validate_pos(@_,
                 1,
                 { type => SCALAR },
                 { type => SCALAR },
                 { can => [ 'handleEvent' ]}
+                { type => SCALAR | UNDEF, optional => 1},
             );
     my $logger = get_logger("perfSONAR_PS::RequestHandler");
 
@@ -428,8 +432,11 @@ sub handleMessage($$$) {
                 $found_md = 1;
 
                 my $eventType;
+		my $found_event_type = 0;
                 my $eventTypes = find($m, "./nmwg:eventType", 0);
+                my $found_event_type = 0;
                 foreach my $e ($eventTypes->get_nodelist) {
+                    $found_event_type = 1;
                     my $value = extract($e, 1);
                     if ($self->isValidEventType($messageType, $value)) {
                         $eventType = $value;
@@ -439,7 +446,7 @@ sub handleMessage($$$) {
 
                 my $errorEventType;
                 my $errorMessage;
-                if (!defined $eventType) {
+                if (($found_event_type && !defined $eventType) or (!$self->isValidMessageType($messageType))) {
                     $errorEventType = "error.ma.event_type";
                     $errorMessage = "No supported event types for message of type \"$messageType\"";
                 } else {
